@@ -7,7 +7,7 @@ Options:
   -h --help                   Show this screen.
   -c FILE --config-file=FILE  Path to a YAML configuration file [default: config.yaml].
   -p INT --period INT         Period length, in minutes (default: 1)
-  -n INT                      Number of data points to get [default: 5]
+  -n INT                      Number of data points to try to get (default: 5)
   -v                          Verbose
 """
 from __future__ import unicode_literals
@@ -31,8 +31,13 @@ else:
 # configuration
 
 DEFAULT_REGION = 'us-east-1'
-DEFAULT_FORMAT = ('cloudwatch.%(Namespace)s.%(dimension)s.%(MetricName)s'
-    '.%(statistic)s.%(Unit)s')
+
+DEFAULT_OPTIONS = {
+    'Period': 1,  # 1 minute
+    'Count': 5,  # 5 periods
+    'Formatter': ('cloudwatch.%(Namespace)s.%(dimension)s.%(MetricName)s'
+        '.%(statistic)s.%(Unit)s')
+}
 
 
 def get_config(config_file):
@@ -41,7 +46,7 @@ def get_config(config_file):
         try:
             return yaml.load(fp)
         except yaml.YAMLError as e:
-            sys.stderr.write(text_type(e))  # XXX python3
+            sys.stderr.write(text_type(e))
             sys.exit(1)  # TODO document exit codes
 
     if config_file == '-':
@@ -50,15 +55,34 @@ def get_config(config_file):
         return load(fp)
 
 
-def output_results(results, metric):
+def get_options(config_options, local_options, cli_options):
+    """
+    Figure out what options to use based on the four places it can come from.
+
+    Order of precedence:
+    * cli_options      specified by the user at the command line
+    * local_options    specified in the config file for the metric
+    * config_options   specified in the config file at the base
+    * DEFAULT_OPTIONS  hard coded defaults
+    """
+    options = DEFAULT_OPTIONS.copy()
+    if config_options is not None:
+        options.update(config_options)
+    if local_options is not None:
+        options.update(local_options)
+    if cli_options is not None:
+        options.update(cli_options)
+    return options
+
+
+def output_results(results, metric, options):
     """
     Output the results to stdout.
 
     TODO: add AMPQ support for efficiency
     """
-    options = metric.get('Options', {})
-    formatter = options.get('Formatter', DEFAULT_FORMAT)
-    context = metric.copy()
+    formatter = options['Formatter']
+    context = metric.copy()  # XXX might need to sanitize this
     try:
         context['dimension'] = list(metric['Dimensions'].values())[0]
     except AttributeError:
@@ -79,28 +103,30 @@ def output_results(results, metric):
             sys.stdout.write(line)
 
 
-def leadbutt(config_file, period, count, verbose=False, **kwargs):
+def leadbutt(config_file, cli_options, verbose=False, **kwargs):
     config = get_config(config_file)
+    config_options = config.get('Options')
+    auth_options = config.get('Auth', {})
 
-    # TODO use auth from config if exists
-    region = config.get('region', DEFAULT_REGION)
+    region = auth_options.get('region', DEFAULT_REGION)
     connect_args = {
         'debug': 2 if verbose else 0,
     }
+    if 'aws_access_key_id' in auth_options:
+        connect_args['aws_access_key_id'] = auth_options['aws_access_key_id']
+    if 'aws_secret_access_key' in auth_options:
+        connect_args['aws_secret_access_key'] = auth_options['aws_secret_access_key']
     conn = boto.ec2.cloudwatch.connect_to_region(region, **connect_args)
-    for metric in config['metrics']:
-        local_options = metric.get('Options', {})
-        # UGH this option checking/fallback code is so ugly
-        if period is None:
-            # If user did not specify period, check config, then use default
-            periods = local_options.get('Period', 1)
-        else:
-            periods = period
-        period_seconds = periods * 60
+    for metric in config['Metrics']:
+        options = get_options(
+            config_options, metric.get('Options'), cli_options)
+        period_local = options['Period'] * 60
+        count_local = options['Count']
         end_time = datetime.datetime.utcnow()
-        start_time = end_time - datetime.timedelta(seconds=period_seconds * count)
+        start_time = end_time - datetime.timedelta(
+            seconds=period_local * count_local)
         results = conn.get_metric_statistics(
-            period_seconds,  # minimum: 60
+            period_local,  # minimum: 60
             start_time,
             end_time,
             metric['MetricName'],  # RequestCount, CPUUtilization
@@ -109,8 +135,8 @@ def leadbutt(config_file, period, count, verbose=False, **kwargs):
             dimensions=metric['Dimensions'],
             unit=metric['Unit'],  # Count, Percent
         )
-        # sys.stderr.write('{} {}\n'.format(count, len(results)))
-        output_results(results, metric)
+        # sys.stderr.write('{} {}\n'.format(options['Count'], len(results)))
+        output_results(results, metric, options)
 
 
 def main(*args, **kwargs):
@@ -118,11 +144,14 @@ def main(*args, **kwargs):
     # help: http://boto.readthedocs.org/en/latest/ref/cloudwatch.html#boto.ec2.cloudwatch.CloudWatchConnection.get_metric_statistics
     config_file = options.pop('--config-file')
     period = options.pop('--period')
-    if period is not None:
-        period = int(period) * 60
-    count = int(options.pop('-n'))
+    count = options.pop('-n')
     verbose = options.pop('-v')
-    leadbutt(config_file, period, count, verbose, **options)
+    cli_options = {}
+    if period is not None:
+        cli_options['Period'] = int(period)
+    if count is not None:
+        cli_options['Count'] = int(count)
+    leadbutt(config_file, cli_options, verbose, **options)
 
 
 if __name__ == '__main__':
